GSecret

Struct GSecret 

pub struct GSecret<V: Validator = NoValidation, const MIN: usize = DEFAULT_MIN, const MAX: usize = DEFAULT_MAX, const ASCII_ONLY: bool = DEFAULT_ASCII_ONLY> { /* private fields */ }
GSecret is a type for containing secret.

Unlike GString, it’s not Copy for security reason.

No Display implementation and debugging is redacted.

It Implements zeroize and can be triggered manually or automatically on drop.

You can reveal the secret inside a closure to do something with it.

If you reveal and clone or take ownership of the revealed string, it goes beyond responsibility of GSecret to zeroize it.

Implementations

Source

impl GSecret

pub fn try_default<S>(secret: S) -> Result<Self, GStringError<Infallible>>
where S: AsRef<str>,

Construct GSecret with default generic params.

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GSecret<V, MIN, MAX, ASCII_ONLY>

pub fn try_new<S>(secret: S) -> Result<Self, GStringError<V::Err>>
where S: AsRef<str>,

Construct GSecret with defined generic params.

pub fn reveal<R>(&self, func: impl FnOnce(&str) -> R) -> R

Reveals the secret.

It’s useful if you want to do something with the secret, e.g. authentication, authorization, etc.

WARNING

Revealing the secret will expose reference to string of the secret itself. As long as you don’t bring it outside of closure scope, it’s fine. If you bring it outside, it goes beyond responsibility of GSecret to zeroize it.

pub fn zeroize(&mut self)

Manually zeroize the secret.

It’s useful if you want to zeroize before the secret is out-of-scope/dopped.