GString

Struct GString 

pub struct GString<V: Validator = NoValidation, const MIN: usize = DEFAULT_MIN, const MAX: usize = DEFAULT_MAX, const ASCII_ONLY: bool = DEFAULT_ASCII_ONLY> { /* private fields */ }
GString contains stack-allocated, Copy, bounded string along with ASCII toggle and embedded validation logic.

Generic parameters:

  • V: Validator: default to NoValidation, it will validate the string upon creation and deserialization.
  • const MIN: usize: default to DEFAULT_MIN, in bytes, determine minimum length.
  • const MAX: usize: default to DEFAULT_MAX, in bytes, determine maximum length.
  • const ASCII_ONLY: bool: default to DEFAULT_ASCII_ONLY, determine whether GString may contains arbitrary or ASCII only UTF-8 encoded string.

Usage

You can use this type in several ways:

  • Defaulted to default generic params: let a: GString = GString::try_new("anjay!!").unwrap().
  • Defaulted to default generic params with try_default(...): let a = GString::try_default("anjay!!").unwrap().
  • Using type aliases. You can declare some aliases matching the behavior you want: type Username = GString<UsernameValidation, 3, 20, true>.
  • Declaring each generic params from left to right. Declaration must be from left to right with omission allowed on right-most params: let a = GString::<(), 2>::try_new("anjay!!").unwrap().

Examples

use g_string::{GString, Validator, GStringError};

let a: GString = GString::try_new("anjay!!").unwrap();
assert_eq!(a, "anjay!!");

let a = GString::try_default("anjay!!").unwrap();
assert_eq!(a, "anjay!!");

#[derive(Debug, Clone)]
struct UsernameValidation;

impl Validator for UsernameValidation {
    type Err = GStringError<&'static str>;

    fn validate(_: impl AsRef<str>) -> Result<(), Self::Err> {
        // some validation logics here...
        Ok(())
    }   
}

type Username = GString<UsernameValidation, 3, 20, true>;
let a = Username::try_new("wanjay!!🤣");
assert!(a.is_err()); // because '🤣' is not ASCII.

let a = GString::<(), 2, 4>::try_new("anjay!!");
assert!(a.is_err()); // MAX is 4.

Implementations

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn try_from_iter<I, S>(iter: I) -> Result<Self, GStringError<V::Err>>
where I: IntoIterator<Item = S>, S: AsRef<str>,

Construct GString from iterator of AsRef<str>.

Examples
use g_string::GString;

let result: GString = GString::try_from_iter(["12", "3a"]).unwrap();
assert_eq!(result, "123a");
pub fn try_from_chars<I>(iter: I) -> Result<Self, GStringError<V::Err>>
where I: IntoIterator<Item = char>,

Construct GString from iterator of chars

Examples
use g_string::GString;

let result: GString = GString::try_from_chars(['a', 'b', 'c']).unwrap();
assert_eq!(result, "abc");
Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn push(&mut self, ch: char) -> Result<(), GStringError<V::Err>>

Push a char at the end of string with invariants preserved.

pub fn push_str(&mut self, s: &str) -> Result<(), GStringError<V::Err>>

Push string at the end of string with invariants preserved.

pub fn insert( &mut self, idx: usize, ch: char, ) -> Result<(), GStringError<V::Err>>

Insert a char at specific index with invariants preserved.

pub fn insert_str( &mut self, idx: usize, string: &str, ) -> Result<(), GStringError<V::Err>>

Insert string at specific index with invariants preserved.

pub fn pop(&mut self) -> Result<Option<char>, GStringError<V::Err>>

Pop a char from the end of string with invariants preserved.

pub fn remove(&mut self, idx: usize) -> Result<char, GStringError<V::Err>>

Removes a char at specific index with invariants preserved.

pub fn truncate(&mut self, new_len: usize) -> Result<(), GStringError<V::Err>>

Truncates to a new length with invariants preserved.

pub fn replace( &mut self, from: &str, to: &str, ) -> Result<(), GStringError<V::Err>>

Replaces from with to in string with invariants preserved.

pub fn replace_range<R>( &mut self, range: R, replace_with: &str, ) -> Result<(), GStringError<V::Err>>
where R: RangeBounds<usize>,

Replaces with range with invariants preserved.

Source

impl<V: Validator + AllowEmpty, const MAX: usize, const ASCII_ONLY: bool> GString<V, 0, MAX, ASCII_ONLY>

pub fn clear(&mut self)

Clear the string, only work if Validator implements AllowEmpty and MIN == 0.

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn try_extend<I, S>(&mut self, iter: I) -> Result<(), GStringError<V::Err>>
where I: IntoIterator<Item = S>, S: AsRef<str>,

Extend existing string with iterator of AsRef<str> with invariants preserved.

Examples
use g_string::GString;

let mut gs: GString<(), 0, 100, false> = GString::try_new("hello").unwrap();
gs.try_extend(["123", "456"].iter().copied());
assert_eq!(gs.as_str(), "hello123456");
pub fn try_extend_chars<I>( &mut self, iter: I, ) -> Result<(), GStringError<V::Err>>
where I: IntoIterator<Item = char>,

Extend existing string with iterator of chars with invariants preserved.

Examples
use g_string::GString;

let mut gs: GString<(), 0, 100, false> = GString::try_new("hello").unwrap();
gs.try_extend_chars(['@', 'z'].iter().copied());
assert_eq!(gs.as_str(), "hello@z");
Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub const fn len(&self) -> usize

Get length of GString in bytes.

pub const fn capacity(&self) -> usize

Tells maximum capacity returning MAX.

pub fn count(&self) -> usize

Counts Unicode scalar values (char).

pub const fn is_empty(&self) -> bool

Check if empty.

pub const fn is_full(&self) -> bool

Tells if maximum capacity met.

pub const fn is_char_boundary(&self, index: usize) -> bool

pub const fn as_str(&self) -> &str

Show GString as &str.

pub const fn as_bytes(&self) -> &[u8]

Show GString as bytes.

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn grapheme_count(&self) -> usize

Counts user-perceived characters (grapheme clusters).

pub fn graphemes(&self) -> Graphemes<'_>

Iterate over graphemes.

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn chars(&self) -> Chars<'_>

Returns an iterator over the chars of a string slice.

pub fn char_indices(&self) -> CharIndices<'_>

Returns an iterator over the chars of a string slice, and their positions.

pub fn bytes(&self) -> Bytes<'_>

Returns an iterator over the bytes of a string slice.

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn contains<P>(&self, pat: P) -> bool
where P: Pattern,

pub fn starts_with<P>(&self, pat: P) -> bool
where P: Pattern,

pub fn ends_with<P>(&self, pat: P) -> bool
where P: Pattern,

pub fn find<P>(&self, pat: P) -> Option<usize>
where P: Pattern,

pub fn rfind<P>(&self, pat: P) -> Option<usize>
where P: Pattern,

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn get<I>(&self, i: I) -> Option<&<I as SliceIndex<str>>::Output>
where I: SliceIndex<str>,

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn split<P: Pattern>(&self, pat: P) -> P::SplitIter<'_>

Returns an iterator over substrings of this string slice, separated by characters matched by a pattern.

pub fn split_whitespace(&self) -> SplitWhitespace<'_>

Returns an iterator over the non-whitespace substrings of this string slice, separated by any amount of whitespace.

pub fn lines(&self) -> Lines<'_>

Returns an iterator over the lines of this string slice.

pub fn split_once<P: Pattern>(&self, delimiter: P) -> Option<(&str, &str)>

Divides this string slice into two at the first occurrence of a pattern.

pub fn rsplit_once<P: Pattern>(&self, delimiter: P) -> Option<(&str, &str)>

Divides this string slice into two at the last occurrence of a pattern.

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn try_trim(&self) -> Result<Self, GStringError<V::Err>>

Returns a trimmed copy of this string.

pub fn try_trim_start(&self) -> Result<Self, GStringError<V::Err>>

Returns a copy of this string with leading whitespace removed.

pub fn try_trim_end(&self) -> Result<Self, GStringError<V::Err>>

Returns a copy of this string with trailing whitespace removed.

pub fn strip_prefix<P: Pattern>(&self, prefix: P) -> Option<&str>

Returns a string slice with the given prefix removed.

pub fn strip_suffix<P: Pattern>(&self, suffix: P) -> Option<&str>

Returns a string slice with the given suffix removed.

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn matches<P: Pattern>(&self, pat: P) -> P::MatchesIter<'_>

Returns an iterator over matches of a pattern within this string slice.

pub fn rmatches<P: Pattern>(&self, pat: P) -> P::RMatchesIter<'_>

Returns an iterator over matches of a pattern within this string slice, yielded in reverse order.

pub fn match_indices<P: Pattern>(&self, pat: P) -> P::MatchIndicesIter<'_>

Returns an iterator over the disjoint matches of a pattern and their positions within this string slice.

pub fn rmatch_indices<P: Pattern>(&self, pat: P) -> P::RMatchIndicesIter<'_>

Returns an iterator over the disjoint matches of a pattern and their positions within this string slice, yielded in reverse order.

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn parse<F>(&self) -> Result<F, F::Err>
where F: FromStr,

Parses this string slice into another type.

pub fn is_ascii(&self) -> bool

Checks if all characters in this string are within the ASCII range.

pub fn eq_ignore_ascii_case(&self, other: &str) -> bool

Compares two strings in a case-insensitive ASCII manner.

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn escape_debug(&self) -> EscapeDebug<'_>

Returns an iterator that escapes this string using char::escape_debug.

pub fn escape_default(&self) -> EscapeDefault<'_>

Returns an iterator that escapes this string using char::escape_default.

pub fn escape_unicode(&self) -> EscapeUnicode<'_>

Returns an iterator that escapes this string using char::escape_unicode.

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn encode_utf16(&self) -> EncodeUtf16<'_>

Returns an iterator of UTF-16 code units for this string slice.

Source

impl GString

pub fn try_default<S>(s: S) -> Result<Self, GStringError<Infallible>>
where S: AsRef<str>,

Construct with default generic params.

Examples
use g_string::GString;

let a = GString::try_default("anjay!!").unwrap();
assert_eq!(a, "anjay!!");
Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn try_new<S>(s: S) -> Result<Self, GStringError<V::Err>>
where S: AsRef<str>,

Construct new GString. All invariants from generic params will be imposed here.

Examples
use g_string::GString;

let a: GString = GString::try_new("anjay!!").unwrap();
assert_eq!(a, "anjay!!");
pub fn validate(self) -> Result<Self, GStringError<V::Err>>

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub const fn new(s: &str) -> InValidatedGString<V, MIN, MAX, ASCII_ONLY>

Construct GString in const context returning invalidated string. Validate the return to get GString.

Source

impl<V: Validator, const MIN: usize, const MAX: usize, const ASCII_ONLY: bool> GString<V, MIN, MAX, ASCII_ONLY>

pub fn zeroize(&mut self)